
#include <afxwin.h>
#include "resources.h"
#include <stdlib.h>
#include <afxmt.h>
#include <process.h>
#include <afx.h>
#include <time.h>

#define LARGURA 31 // Largura da casa menos 1
#define ALTURA 31
#define LINHAS 16
#define COLUNAS 17

#define PONTOS_INIMIGO 5000
#define PONTOS_BOLA 1000
#define PONTOS_PODER 10000
#define PONTOS_PARA_GANHAR_VIDA 500000

#define MAX_JOGADORES 2
#define MAX_INIMIGOS 64

#define BORDA_SUPERIOR 10
#define BORDA_ESQUERDA 10
#define BORDA_INFERIOR 10
#define BORDA_DIREITA 240
#define BORDA 125 //Utilizada somente no Intro.cpp

#define MARCADA 1
#define VAO 0
#define PAREDE 1
#define PODER 3
#define BOLA 2

enum TSentido {sCima,sBaixo,sDireita,sEsquerda,sNenhum}; //Sentido de movimento
enum TStatus {sNormal, sAnormal, sMorto}; //Estado dos personagens

typedef unsigned (WINAPI *CAST_FUNCTION)(LPVOID); //casting necessrios
typedef unsigned *CAST_LPDWORD ;                 //para criar as threads

typedef char TMatriz [LINHAS][COLUNAS]; 
typedef struct	{
				int NormalA,NormalB,AnormalA,AnormalB;
				} TSkinPersonagem;
typedef struct {
				int Parede,Bola,Poder,Vao;
				} TSkinLabirinto;
typedef struct {
	int Cima,Baixo,Esquerda,Direita;
}TTeclado; //Mapear o teclado


#define MAX_HANDLES 128 //Mximo de handles criados no programa
class CHandle
// Classe para controlar os handles alocados
{
private:
	int N;
	CCriticalSection SecaoCriticaHandle;
	HANDLE hVetor[MAX_HANDLES]; //Guarda a handle da thread
	DWORD idVetor[MAX_HANDLES]; //Guarda a ID da thread

public:
	bool AdicionaHandle(HANDLE h,DWORD id);
	BOOL FechaHandle(HANDLE h);
	BOOL FechaHandle(DWORD ID);
	bool FechaTodosHandles();
	bool EsperePorTodosObjetos(DWORD Milisegundos);
};
//-----------------------------------------------------------------------------
class CLabirinto; //Forward de CLabirinto
//Definio em "Personagem.cpp"
class CPersonagem
{
private:
	CBitmap *SkinNormalA; CBitmap *SkinNormalB;
	CBitmap *SkinAnormalA; CBitmap *SkinAnormalB;
	bool UsarSkinA;	int ContSkinA; //Auxilio grfico
protected:	
	HANDLE hEvento;
	TStatus Status;
	int TempoDeEspera;
	TSentido SentidoAtual;
	CCriticalSection *pSecaoCritica;
public:
	void CriaPersonagem(POINT Pos, TSkinPersonagem Skin,
		                CLabirinto *pLab,CCriticalSection *pSC,
						int TempoEspera);
	void CarregaBitmaps(TSkinPersonagem Skin);
	POINT Posicao;
	void MudaStatus(TStatus NovoStatus);
	TStatus GetStatus();
	void Desenha(CDC *pDC);
	void SetTempo(int Tempo);
	CLabirinto *pLabirinto;
	bool Move(TSentido Sentido);
	POINT GetPosicao();
	void ForcaPosicao(POINT P);
	void MorreDesgracado();
	void SetaEvento();
	void ResetaEvento();
	DWORD ID;
	~CPersonagem();	
};
//-----------------------------------------------------------------------------
//Definio em "Jogador.cpp"
class CJogador : public CPersonagem
{
private:
	int Pontos;
	
	int Vidas;
	int NInimigosMortos;
	TSentido SentidoManual;
	char TeclaCima,TeclaBaixo,TeclaEsquerda,TeclaDireita;
	time_t TempoAnormal;
public:
	void TomaAtitude();
	void AguardeComando();
	void Tecla(char Key);
	void MapeiaTeclado(TTeclado Teclas);
	int GetVidas();
	int GetInimigosMortos();
	int GetPontos();
	CJogador();
};
//-----------------------------------------------------------------------------
//Definio em "Inimigo.cpp"
class CInimigo : public CPersonagem
{
private:
	int Tempo;
public:
	void Vagueia();
};
//-----------------------------------------------------------------------------
//Definio em "Labirino.cpp"
class CLabirinto
{
private:
	TMatriz Casas;
	CBitmap *BitmapVao;
	CBitmap *BitmapParede;
	CBitmap *BitmapBola;
	CBitmap *BitmapPoder;
	TSkinPersonagem SkinHeroi,SkinInimigo;
	int NJogadores;
	int NInimigos;
	int TotalDeBolas;
	int TempoInimigo,TempoJogador;
	CCriticalSection *pSecaoCritica;
	HANDLE hMorteMorrida;
	CHandle *pHandlesInimigos;
	CHandle *pHandlesJogadores;

public:
	CJogador *Jogadores[MAX_JOGADORES];
	CInimigo *Inimigos[MAX_INIMIGOS];
	void DesenhaLabirinto(CDC *pDC);
	void CarregaLabirinto(TMatriz mat);
	char GetPosicao(POINT pos);
	CJogador* GetJogador(POINT pos);
	CInimigo* GetInimigo(POINT pos);
	CJogador* GetJogador(int Indice);
	CInimigo* GetInimigo(int Indice);
	void SetPHandles(CHandle *phJogadores, CHandle *phInimigos);
	void RemoveItem(POINT pos);
	void RemoveBoot(CInimigo *p);
	void RemoveJogador(CJogador *p);
	int AdicionaInimigo(TSkinPersonagem Skin);
	bool AdicionaJogador(TSkinPersonagem Skin,TTeclado Teclas,int Indice);
	void CarregaBitmaps(TSkinLabirinto SkinLabirinto,
		TSkinPersonagem SkinDoHeroi, TSkinPersonagem SkinDoInimigo);
	TSkinPersonagem GetSkinHeroi();
	TSkinPersonagem GetSkinInimigo();
	void EnfraqueceInimigos();
	void FortaleceInimigos();
	void RecolheJogadores();
	void RecolheInimigos();
	void SetTempos(int iTempoInimigo, int iTempoJogador);
	void ChacinaInimigos();
	int GetTotalDeBolas();
	void Tecla(char Key);
	CLabirinto();
	~CLabirinto();
};

//Funes extras fora das classes
POINT POS(int x, int y); //Facilitar a passagem de um ponto
void CriaThreadInimigo(CInimigo *p,CHandle* pHandles); // 1
void CriaThreadJogador(CJogador *p,CHandle* pHandles); // 1
DWORD WINAPI ExecutaThreadJogador(CJogador *p);        // 1
DWORD WINAPI ExecutaThreadInimigo(CInimigo *p);        // 1
DWORD WINAPI RefrescaTela();  // 2
void PassaDificuldade(int N); // 2
void PassaFase(int F);    // 2

//1 --> Declaradas em "threads.cpp"
//2 --> Declaradas em "JanelaPrincipal.cpp"