/******************************************************************************
 *   HANDLE.CPP                                                               *
 ******************************************************************************		
 *                                                                            *
 *    Arquivo contendo as definies da classe CHandle, que tm o objetivo de *
 * controlar a incluso e excluso (com respectivos fechamentos) dos handles  *
 * provenientes das threads em execuo                                       *
 *                                                                            *
 ******************************************************************************
 *                                                                            * 
 *    Trabalho Prtico de Automao em Tempo Real                             *
 *       6 Perodo de Engenharia de Controle e Automao                     *
 *           Universidade Federal de Minas Gerais                             *
 *                                                                            *
 *    Prof.:  Constantino Seixas Filho                                        *
 *                                                                            *
 *    Alunos: Rafael Cervolo                                                 *
 *            Thiago Barbosa                                                  *
 *                                                                            *
 *****************************************************************************/


 // A SecaoCriticaHandle controla a modificao do vetor de handles.
 // Apesar de ser um mesmo objeto, vrias instancias das fues desta classe
 // podem estar sendo acessadas concorrentemente pelas threads em execuo.

#include "classes.h" 
//---------------------------------------------------------------------------//
bool CHandle::AdicionaHandle(HANDLE h, DWORD id)
// Adicionar um novo handle  lista.
// RETORNO:  true-> handle adicionado com sucesso
//           false-> no foi possvel adicionar handle, lista cheia!
{
if (N >= MAX_HANDLES) 
	return (false);	// Se no couber mais, falha.
SecaoCriticaHandle.Lock();
	hVetor[N] = h;
	idVetor[N] = id;
	N++;
SecaoCriticaHandle.Unlock();
return (true); //Adio efetuada com sucesso
}
//---------------------------------------------------------------------------//
BOOL CHandle::FechaHandle(DWORD ID)
// Fecha o HANDLE correspondente a thread com identificao ID
// RETORNO:  true -> handle fechado com sucesso
//           false -> handle no encontrado na lista
{
int i,ind=0;
BOOL Ret;
SecaoCriticaHandle.Lock();
for (i = 0;i < N; i++)
{
	if(idVetor[i] == ID)
	{
		Ret = CloseHandle(hVetor[i]);
		if (Ret)
		{
			ind = i;
			N--;
			break;
		}
	}
}
if (Ret) // Se foi fechado, ento "shift left" do vetor na posicao removida
	for (i=ind;i< N;i++) //
	{
		hVetor[i] = hVetor[i+1];
		idVetor[i] = idVetor[i+1];
	}
SecaoCriticaHandle.Unlock();
return (Ret);
}
//---------------------------------------------------------------------------//
BOOL CHandle::FechaHandle(HANDLE h)
// Fecha o HANDLE "h", previamente aberto pelo sistema.
// RETORNO:  true -> handle fechado com sucesso
//           false -> handle no encontrado na lista
{
int i,ind=0;
BOOL Ret;
SecaoCriticaHandle.Lock();
for (i = 0;i < N; i++)
{
	if(hVetor[i] == h)
	{
		Ret = CloseHandle(h);
		if (Ret)
		{
			ind = i;
			N--;
			break;
		}
	}
}
if (Ret) // Se foi fechado, ento "shift left" do vetor na posicao removida
	for (i=ind;i< N;i++) //
		hVetor[i] = hVetor[i+1];
SecaoCriticaHandle.Unlock();
return (Ret);
}
//---------------------------------------------------------------------------//
bool CHandle::FechaTodosHandles()
{
//Fecha todos os handles da lista
// RETORNO:  true -> todos os handle fechado com sucesso
//           false -> pelo menos um deu treta.
bool tudo_ok = true;
SecaoCriticaHandle.Lock();
for (int i = 0;i < N; i++)
	if (!CloseHandle(hVetor[i]))
		tudo_ok = false;
N=0;
SecaoCriticaHandle.Unlock();
return (tudo_ok);
}
//---------------------------------------------------------------------------//
bool CHandle::EsperePorTodosObjetos(DWORD Milisegundos)
// Executa um WaitForMultipleObjects na lista de handles
// RETORNO:  true -> todos os handles retornaram com sucesso
//           false -> nem todos os handles retornaram com sucesso
{
DWORD Ret;
Ret = WaitForMultipleObjects(N,hVetor,TRUE,Milisegundos);
return (Ret == WAIT_OBJECT_0);
}