/******************************************************************************
 *   JANELAPRINCIPAL.CPP                                                                *
 ******************************************************************************		
 *                                                                            *
 * Arquivo contendo a declarao da interface com o usurio, como menus,      *
 * janela principal, teclado etc                                              *
 *                                                                            *
 ******************************************************************************
 *                                                                            * 
 *    Trabalho Prtico de Automao em Tempo Real                             *
 *       6 Perodo de Engenharia de Controle e Automao                     *
 *           Universidade Federal de Minas Gerais                             *
 *                                                                            *
 *    Prof.:  Constantino Seixas Filho                                        *
 *                                                                            *
 *    Alunos: Rafael Cervolo                                                 *
 *            Thiago Barbosa                                                  *
 *                                                                            *
 *****************************************************************************/
#include "classes.h"
#include "janelaprincipal.h"

//---------------------------------------------------------------------------//
CJogador::CJogador()
{
	Pontos = 0;
	Vidas = 3;
	NInimigosMortos = 0;
	SentidoManual = sNenhum;
	SentidoAtual = sNenhum;
}
//---------------------------------------------------------------------------//
void CJogador::TomaAtitude()
{
// TomaAtitude toma atitude sobre a posio atual do Jogador:
// NOTA: A chamada desta funo deve ser protegida com uma seo crtica.
	switch(pLabirinto->GetPosicao(Posicao))
	{
	case BOLA:
		// Se tiver bola, 
		Pontos+=PONTOS_BOLA; //ganha ponto
		pLabirinto->RemoveItem(Posicao); //remove do labirinto
		if (Pontos%PONTOS_PARA_GANHAR_VIDA == 0) Vidas++;
		break;
	case PODER: // Se tiver poder
		Pontos+=PONTOS_INIMIGO; //ganha mais ainda
		pLabirinto->EnfraqueceInimigos(); 
		Status = sAnormal;
		pLabirinto->RemoveItem(Posicao);
		time(&TempoAnormal); //marca o tempo que ficou vitaminado
		break;
	}
	CInimigo *p;
	TStatus StatusDoInimigo;
	while ((p = pLabirinto->GetInimigo(Posicao)) != NULL)
	{ 
		if (Status == sAnormal) //Se tiver poderoso
		{
			StatusDoInimigo = p->GetStatus();
			if (StatusDoInimigo==sAnormal)
			{
				//Se ele estiver fraco, mata ele!
				p->MorreDesgracado();
				NInimigosMortos ++;
				Pontos+=PONTOS_PODER;
			}
			else
			{
				//Caso contrrio, mesmo se o jogador estiver forte,
				// o inimigo tambm forte  mais forte!
				Status = sMorto;
				break;
			}
		}
		else
		{
			//Se no tiver poderoso, no tem nem chance!
			Status = sMorto;
			break;
		}	
	}
}
//------------------------------------------------------------------------------//
void CJogador::AguardeComando()
{
// Esta  no final das contas a funo que rodar indefinidamente
// na thread do Jogador.
time_t TempoAtual;
while (Vidas > 0)
{
	WaitForSingleObject(hEvento,TempoDeEspera);
	//Espera uma sinalizao ou por um time-out, que  o tempo de 
	// auto-movimento
	pSecaoCritica->Lock();
	if (Status == sAnormal)
	{
		time(&TempoAtual); //Pega o tempo atual
		if ((TempoAtual-TempoAnormal) >= 10) // Se tiver 10 ou mais segundos
											// que est poderoso, fica fraco.
		{
			Status = sNormal;
			pLabirinto->FortaleceInimigos();
		}
	}
	if (Status == sMorto)
	{
		Vidas--; 
		Status = sNormal; 
		pLabirinto->RecolheInimigos();
		pLabirinto->RecolheJogadores();
		SentidoAtual = sNenhum;
		pLabirinto->FortaleceInimigos();
		ResetaEvento(); //Como o inimigo que me matou deve ter sinalizado
		                //o evento, eu reseto ela.
	}
	else if(Move(SentidoManual))
	{
		// Se conseguiu mover, 
		TomaAtitude(); //ento v que que aconteceu
		SentidoAtual = SentidoManual; 
		SentidoManual = sNenhum;
	}
	else if(Move(SentidoAtual))
	{
		TomaAtitude();
	}
	else
	{
		SentidoAtual = sNenhum;
	}
	pSecaoCritica->Unlock();
	if (pLabirinto->GetTotalDeBolas() == 0)
	{
		//pLabirinto->ChacinaInimigos();
		AvancaFase();
	}
}
pLabirinto->RemoveJogador(this); //Se morreu, ento remove o danado.
_endthreadex((DWORD) 0);
}
//---------------------------------------------------------------------------//
void CJogador::Tecla(char Key)
{
// V se a tecla passada corresponde ao mapa:
if(Key == TeclaCima)
	SentidoManual = sCima;
else if (Key == TeclaBaixo)
	SentidoManual = sBaixo;
else if (Key == TeclaEsquerda)
	SentidoManual = sEsquerda;
else if (Key == TeclaDireita)
	SentidoManual = sDireita;
}
//---------------------------------------------------------------------------//
void CJogador::MapeiaTeclado(TTeclado Teclas)
{
TeclaEsquerda = Teclas.Esquerda;
TeclaDireita = Teclas.Direita;
TeclaCima = Teclas.Cima;
TeclaBaixo = Teclas.Baixo;
}
//---------------------------------------------------------------------------//
int CJogador::GetInimigosMortos()
{
return (NInimigosMortos);
}
int CJogador::GetPontos()
{
return (Pontos);
}
int CJogador::GetVidas()
{
return (Vidas);
}
//---------------------------------------------------------------------------//


