/******************************************************************************
 *   LABIRINTO.CPP                                                            *
 ******************************************************************************		
 *                                                                            *
 * A classe labirinto  responsavel no s pelo contedo do labirinto, mas    *
 * serve como porto de acesso aos personagens e os controles do jogo         *
 *                                                                            *
 ******************************************************************************
 *                                                                            * 
 *    Trabalho Prtico de Automao em Tempo Real                             *
 *       6 Perodo de Engenharia de Controle e Automao                     *
 *           Universidade Federal de Minas Gerais                             *
 *                                                                            *
 *    Prof.:  Constantino Seixas Filho                                        *
 *                                                                            *
 *    Alunos: Rafael Cervolo                                                 *
 *            Thiago Barbosa                                                  *
 *                                                                            *
 *****************************************************************************/

#include "classes.h"
#include "janelaprincipal.h"

POINT POS(int x, int y)
{
	POINT aux;
	aux.x=x;
	aux.y=y;
	return (aux);
}
//---------------------------------------------------------------------------//
CLabirinto::~CLabirinto()
{
//Destrutor do CLabirinto
int i;
for (i=0;i<MAX_JOGADORES;i++)
	if (Jogadores[i]!= NULL)
		delete (Jogadores[i]);
for (i=0;i<NInimigos;i++)
	delete (Inimigos[i]);
delete (BitmapBola);
delete (BitmapParede);
delete (BitmapPoder);
delete (BitmapVao);
}
//---------------------------------------------------------------------------//
CLabirinto::CLabirinto()
//Construtor
{
pSecaoCritica = new(CCriticalSection);
//hMorteMorrida = CreateEvent(NULL,true,false,NULL);
NInimigos = 0;
NJogadores = 0;
int i;
for (i=0;i<MAX_JOGADORES;i++)
{
	Jogadores[i] = NULL;
}
for (i=0;i<MAX_INIMIGOS;i++)
{
	Inimigos[i] = NULL;
}
}
//---------------------------------------------------------------------------//
void CLabirinto::DesenhaLabirinto (CDC *pDC)
{
int i,j;
int esq,cima;
TMatriz m;

POINT pt;
/* Para que a imagem no pisque, o labirinto  desenhado apenas onde no h 
 personagens, para saber onde , marca uma matriz temporria*/

for (i=0; i<LINHAS; i++)
	for (j=0; j<COLUNAS; j++)
		m[i][j]=0;

pSecaoCritica->Lock();
	// Depois desenha os personagens
	for (i=0;i<NInimigos;i++)
	{
		Inimigos[i]->Desenha(pDC);
		pt=Inimigos[i]->GetPosicao();
		m[pt.y][pt.x]=1; //aproveita para marcar a posicao
	}
	for (i=0;i<MAX_JOGADORES;i++)
	{
		if (Jogadores[i]!= NULL)
		{
			Jogadores[i]->Desenha(pDC); 
			pt=Jogadores[i]->GetPosicao();
			m[pt.y][pt.x]=1;
		}
	}
	for (i=0; i<LINHAS; i++)
	for (j=0; j<COLUNAS; j++)
	if (m[i][j]!=1)
	{
		//Aqui desenha o labirinto
		esq = j*(LARGURA) + BORDA_ESQUERDA;
		cima = i*(ALTURA) + BORDA_SUPERIOR;
		switch (Casas[i][j])
		{
		case PAREDE:
			pDC->DrawState(CPoint(esq,cima),CSize(LARGURA,ALTURA),
				BitmapParede,DST_BITMAP,NULL);
			break;
		case VAO:
			pDC->DrawState(CPoint(esq,cima),CSize(LARGURA,ALTURA),
				BitmapVao,DST_BITMAP,NULL);
			break;
		case BOLA:
			pDC->DrawState(CPoint(esq,cima),CSize(LARGURA,ALTURA),
				BitmapBola,DST_BITMAP,NULL);
			break;
		case PODER:
			pDC->DrawState(CPoint(esq,cima),CSize(LARGURA,ALTURA),
				BitmapPoder,DST_BITMAP,NULL);
			break;
		}
	}
pSecaoCritica->Unlock();
}
//---------------------------------------------------------------------------//
void CLabirinto::Tecla(char Key)
{
int i;
pSecaoCritica->Lock();
	for (i=0;i<MAX_JOGADORES;i++)
	{
		if (Jogadores[i]!= NULL)
		{
			Jogadores[i]->Tecla(Key);
		}
	}
pSecaoCritica->Unlock();
}
//---------------------------------------------------------------------------//
void CLabirinto::SetPHandles(CHandle *phJogadores, CHandle *phInimigos)
{
	pHandlesJogadores = phJogadores;
	pHandlesInimigos = phInimigos;
}
//---------------------------------------------------------------------------//
void CLabirinto::CarregaLabirinto(TMatriz mat)
{
TotalDeBolas = 0;
	for (int i=0; i<LINHAS; i++)
	for (int j=0; j<COLUNAS; j++)
	{
		Casas[i][j] = mat[i][j];
		if (Casas[i][j] == BOLA)
			TotalDeBolas++;
	}
}
//---------------------------------------------------------------------------//
void CLabirinto::CarregaBitmaps(TSkinLabirinto SkinLabirinto,
		TSkinPersonagem SkinDoHeroi, TSkinPersonagem SkinDoInimigo)
{
pSecaoCritica->Lock();
delete (BitmapVao);
delete (BitmapParede);
delete (BitmapBola);
delete (BitmapPoder);
BitmapVao = new(CBitmap);
BitmapParede = new(CBitmap);
BitmapBola = new(CBitmap);
BitmapPoder = new(CBitmap);

BitmapVao->LoadBitmap(SkinLabirinto.Vao);
BitmapParede->LoadBitmap(SkinLabirinto.Parede);
BitmapBola->LoadBitmap(SkinLabirinto.Bola);
BitmapPoder->LoadBitmap(SkinLabirinto.Poder);

SkinHeroi = SkinDoHeroi;
SkinInimigo = SkinDoInimigo;

int i;
for (i=0;i<MAX_JOGADORES;i++)
	if (Jogadores[i]!= NULL)
		Jogadores[i]->CarregaBitmaps(SkinHeroi);
for (i=0;i<NInimigos;i++)
	Inimigos[i]->CarregaBitmaps(SkinInimigo);
pSecaoCritica->Unlock();
}
//---------------------------------------------------------------------------//
char CLabirinto::GetPosicao(POINT pos)
{
// Retorna o contedo da casa da posio "pos"
return (Casas[pos.y][pos.x]);
}
//---------------------------------------------------------------------------//
TSkinPersonagem CLabirinto::GetSkinHeroi()
{
//Retorna o skin do heroi
return (SkinHeroi);
}
//---------------------------------------------------------------------------//
TSkinPersonagem CLabirinto::GetSkinInimigo()
{
//Retorna o skin do inimigo
return (SkinInimigo);
}
//---------------------------------------------------------------------------//
void CLabirinto::RemoveItem(POINT pos)
{
//Substitui a posio por um espao vazio
if (Casas[pos.y][pos.x] == BOLA)
{	
	TotalDeBolas --;
}
	Casas[pos.y][pos.x] = VAO;

}
//---------------------------------------------------------------------------//
int CLabirinto::AdicionaInimigo(TSkinPersonagem Skin)
{
// O valor de retorno  a posio onde o inimigo foi adicionado
// O vetor  mantido de modo que as NInimigos posies iniciais fiquem ocupadas

if (NInimigos < MAX_INIMIGOS)
{
	pSecaoCritica->Lock();
	Inimigos[NInimigos] = new (CInimigo);
	Inimigos[NInimigos]->CriaPersonagem(POS(8,6),Skin,this,pSecaoCritica,500);
	NInimigos++;	
	pSecaoCritica->Unlock();
	return(NInimigos-1);	

}
return(-1);
}
//---------------------------------------------------------------------------//
bool CLabirinto::AdicionaJogador(TSkinPersonagem Skin, TTeclado Teclas, int Indice)
{
// A incluso do jogador  um pouco diferente:
// A sua posio no vetor  posta numa posio especificada por Indice
// Isto se deve para se ter acesso direto aos comandos dos usurios
if (Indice < MAX_JOGADORES)
{
	if (Jogadores[Indice] == NULL)
	{
		Jogadores[Indice] = new (CJogador);
		Jogadores[Indice]->CriaPersonagem(POS(8,8),Skin,this,pSecaoCritica,TempoJogador);
		Jogadores[Indice]->MapeiaTeclado(Teclas);
		NJogadores++;
		return(true);	
	}
}
return(false);
}
//---------------------------------------------------------------------------//
CJogador* CLabirinto::GetJogador(POINT pos)
{
// Retorna ponteiro para o jogador com posicao = pos
int i;
POINT p;
for (i=0;i<MAX_JOGADORES;i++)
	if (Jogadores[i]!= NULL)
	{
		p = Jogadores[i]->GetPosicao();
		if (p.x == pos.x && p.y == pos.y)
			return(Jogadores[i]);
	}
return (NULL);
}
//---------------------------------------------------------------------------//
CInimigo* CLabirinto::GetInimigo(POINT pos)
{
// Retorna ponteiro para o inimigo com posicao = pos
int i;
POINT p;
for (i=0;i<NInimigos;i++)
{
	p = Inimigos[i]->GetPosicao();
	if (p.x == pos.x && p.y == pos.y && Inimigos[i]->GetStatus()!=sMorto)
		return(Inimigos[i]);
}
return (NULL);
}
//---------------------------------------------------------------------------//
CJogador* CLabirinto::GetJogador(int Indice)
{
// Retorna ponteiro para jogador da posio Indice
if (Indice >= MAX_JOGADORES)
	return (NULL);
else
	return Jogadores[Indice];
}
//---------------------------------------------------------------------------//
CInimigo* CLabirinto::GetInimigo(int Indice)
{
//Retorna ponteiro para o inimigo da posio Indice
if (Indice >= NInimigos)
	return (NULL);
else
	return Inimigos[Indice];
}
//---------------------------------------------------------------------------//
void CLabirinto::RemoveJogador(CJogador *p)
{
int i;
for (i=0;i<MAX_JOGADORES;i++)
{
	if (Jogadores[i]==p)
	{
		pHandlesJogadores->FechaHandle(p->ID);
		delete (Jogadores[i]);
		Jogadores[i]=NULL;
		NJogadores--;
	}
}
}
//---------------------------------------------------------------------------//
void CLabirinto::RemoveBoot(CInimigo *p)
{
int i,j;
pSecaoCritica->Lock();
for (i=0;i<NInimigos;i++)
{
	if (Inimigos[i]==p)
	{
		pHandlesInimigos->FechaHandle(p->ID);
		delete (Inimigos[i]);
		NInimigos--;
		for (j=i;j<NInimigos;j++)
			Inimigos[j]=Inimigos[j+1];
	}
}
pSecaoCritica->Unlock();
}
//---------------------------------------------------------------------------//
void CLabirinto::EnfraqueceInimigos()
{
//Muda o Status de todos os inimigos para sAnormal
// NOTA: Deve ser protegida com excluso mtua
int i;
for (i=0;i<NInimigos;i++)
{
	Inimigos[i]->MudaStatus(sAnormal);
}
}
//---------------------------------------------------------------------------
void CLabirinto::FortaleceInimigos()
{
int i;
for (i=0;i<MAX_JOGADORES;i++)
{
	if (Jogadores[i]!= NULL)
	{
		//Se tiver outro jogador ainda forte, os inimigos continuam fracos
		if (Jogadores[i]->GetStatus() == sAnormal)
			return;
	}
}
for (i=0;i<NInimigos;i++)
{
	Inimigos[i]->MudaStatus(sNormal);
}
}
//----------------------------------------------------------------------------
void CLabirinto::SetTempos(int iTempoInimigo, int iTempoJogador)
{
int i;
// Registra os tempos que os personagens usaram para seus temporizadores
TempoInimigo = iTempoInimigo;
TempoJogador = iTempoJogador;

for (i=0;i<MAX_JOGADORES;i++)
	if (Jogadores[i]!= NULL)
		Jogadores[i]->SetTempo(TempoJogador);
for (i=0;i<NInimigos;i++)
	Inimigos[i]->SetTempo(TempoInimigo);
}
//----------------------------------------------------------------------------
void CLabirinto::ChacinaInimigos()
{
// Mata todos os inimigos
pSecaoCritica->Lock();
int i;
for (i=0;i<NInimigos;i++)
{
	Inimigos[i]->MorreDesgracado();
}
pSecaoCritica->Unlock();
// Espera todos estarem mortos:
pHandlesInimigos->EsperePorTodosObjetos(INFINITE);
}
//----------------------------------------------------------
int CLabirinto::GetTotalDeBolas()
{
return(TotalDeBolas);
}
//-----------------------------------
void CLabirinto::RecolheJogadores()
{
// Coloca os jogadores nas posies iniciais
int i;
for (i=0;i<MAX_JOGADORES;i++)
	if (Jogadores[i]!= NULL)
		Jogadores[i]->ForcaPosicao(POS(8,8));
}
//------------------------------------
void CLabirinto::RecolheInimigos()
{
//Coloca os inimigos nas posies iniciais
for (int i=0;i<NInimigos;i++)
{
	Inimigos[i]->ForcaPosicao(POS(8,6));
}
}