/******************************************************************************
 *   PERSONAGEM.CPP                                                           *
 ******************************************************************************		
 *                                                                            *
 * Classe com os comandos e caractersticas comum dos personagem              *
 * tanto os Heris quanto os Inimigos                                         *
 *                                                                            *
 ******************************************************************************
 *                                                                            * 
 *    Trabalho Prtico de Automao em Tempo Real                             *
 *       6 Perodo de Engenharia de Controle e Automao                     *
 *           Universidade Federal de Minas Gerais                             *
 *                                                                            *
 *    Prof.:  Constantino Seixas Filho                                        *
 *                                                                            *
 *    Alunos: Rafael Cervolo                                                 *
 *            Thiago Barbosa                                                  *
 *                                                                            *
 *****************************************************************************/

#include "Classes.h"
//---------------------------------------------------------------------------//
CPersonagem::~CPersonagem()
{
//Destrutor
delete (SkinAnormalA);
delete (SkinAnormalB);
delete (SkinNormalA);
delete (SkinNormalB);
CloseHandle(hEvento);
}
//---------------------------------------------------------------------------//
bool CPersonagem::Move(TSentido Sentido)
// Movimento o personagem na direo indicada por Sentido.
// Se no for possvel, retorna falso.
{
POINT proximo;
switch (Sentido)
{
case sNenhum:
	return(false);
case sCima:
	proximo.x = Posicao.x;
	proximo.y = Posicao.y-1;
	if (proximo.y < 0)
		return (false);
	break;
case sBaixo:
	proximo.x = Posicao.x;
	proximo.y = Posicao.y+1;
	if (proximo.y >= LINHAS)
		return (false);
	break;
case  sDireita:
	proximo.x = Posicao.x+1;
	proximo.y = Posicao.y;
	if (proximo.x >= COLUNAS)
		return (false);
	break;
case sEsquerda:
	proximo.x = Posicao.x-1;
	proximo.y = Posicao.y;
	if (proximo.x < 0)
		return (false);
	break;		
}
if (pLabirinto->GetPosicao(proximo) == PAREDE)
{
	return (false);
}
Posicao = proximo;
return(true);
}
//---------------------------------------------------------------------------//
void CPersonagem::CriaPersonagem(POINT Pos, TSkinPersonagem Skin,
		                CLabirinto *pLab,CCriticalSection *pSC,
						int TempoEspera)
{
	//Aps criar um CPersonagem, suas caractersticas iniciais devem
	// ser passada por esta funo
	pLabirinto = pLab;
	Posicao = Pos;
	SkinNormalA = new (CBitmap);
	SkinNormalA->LoadBitmap(Skin.NormalA);
	SkinAnormalA = new (CBitmap);
	SkinAnormalA->LoadBitmap(Skin.AnormalA);
	SkinNormalB = new (CBitmap);
	SkinNormalB->LoadBitmap(Skin.NormalB);
	SkinAnormalB = new (CBitmap);
	SkinAnormalB->LoadBitmap(Skin.AnormalB);
	Status=sNormal;
	hEvento = CreateEvent(NULL,true,false,NULL);
	pSecaoCritica = pSC;
	TempoDeEspera = TempoEspera;

}
//---------------------------------------------------------------------------//
void CPersonagem::CarregaBitmaps(TSkinPersonagem Skin)
{
	delete (SkinNormalA);
	delete (SkinAnormalA);
	delete (SkinNormalB);
	delete (SkinAnormalB);
	SkinNormalA = new (CBitmap);
	SkinNormalA->LoadBitmap(Skin.NormalA);
	SkinAnormalA = new (CBitmap);
	SkinAnormalA->LoadBitmap(Skin.AnormalA);
	SkinNormalB = new (CBitmap);
	SkinNormalB->LoadBitmap(Skin.NormalB);
	SkinAnormalB = new (CBitmap);
	SkinAnormalB->LoadBitmap(Skin.AnormalB);
}
//-------------------------------------------------------------------------
void CPersonagem::Desenha(CDC *pDC)
{
// Desenha o personagem na tela
int esq,cima;
esq = Posicao.x*(LARGURA) + BORDA_ESQUERDA;
cima = Posicao.y*(ALTURA) + BORDA_SUPERIOR;

CBitmap *pBitmap;
CDC DCLocal; //Necessrio para se copiar os bits invertidos, quando
             // estiver movimentando para a esquerda
DCLocal.CreateCompatibleDC(pDC);

ContSkinA++;
if (ContSkinA>4)
{
	UsarSkinA = !UsarSkinA;
	ContSkinA=0;
}
if (Status==sNormal)
{
	if (UsarSkinA)
		pBitmap = SkinNormalA;
	else
		pBitmap = SkinNormalB;
}
else if (Status==sAnormal)
{    
	if (UsarSkinA)
		pBitmap = SkinAnormalA;
	else
		pBitmap = SkinAnormalB;
}
if (SentidoAtual == sEsquerda)
{
	//Se estiver para a esquerda, desenha-se o espelho da imagem padro
	DCLocal.SelectObject(pBitmap);
	pDC->StretchBlt(esq+LARGURA-1,cima,-LARGURA,ALTURA,&DCLocal,0,0,31,31,SRCCOPY);
}
else
    pDC->DrawState(CPoint(esq,cima),CSize(LARGURA,ALTURA),pBitmap,DST_BITMAP,NULL);
}
//------------------------------------------------------------------------------
POINT CPersonagem::GetPosicao()
{
	return (Posicao);
}
//------------------------------------------------------------------------------
void CPersonagem::MudaStatus (TStatus NovoStatus)
{
	Status = NovoStatus;
}
//------------------------------------------------------------------------------
TStatus CPersonagem::GetStatus ()
{
	return(Status);
}
//------------------------------------------------------------------------------
void CPersonagem::MorreDesgracado ()
{
// Por enquanto, para matar o inimigo, basta mudar o Status para Morto
// e esperar que os loops das threads detecten esta nudana e deem um
// ExitThread

Status = sMorto;
SetaEvento(); // Acordar a thread (caso esteja dormindo).
}
//-----------------------------------------------------------------------------
void CPersonagem::SetaEvento()
{
	SetEvent(hEvento);
}
//-----------------------------------------------------------------------------
void CPersonagem::ResetaEvento()
{
	ResetEvent(hEvento);
}
//-----------------------------------------------------------------------------
void CPersonagem::SetTempo(int Tempo)
{
TempoDeEspera = Tempo;
}
//-----------------------------------------------------------------------------
void CPersonagem::ForcaPosicao(POINT P)
{
	Posicao.x = P.x;
	Posicao.y = P.y;
}
//-----------------------------------------------------------------------------
